import numpy as np
from scipy.ndimage import map_coordinates
from .utilities import rotate_coordinates
from .filtering import ramp_filter_and_weight
from .config import Config


def map_object_to_detector_coords(object_xs, object_ys, object_zs, settings):
    """Map the object coordinates to detector pixel coordinates accounting for cone beam divergence.

        This implementation supports both a axis-symmetric mode and full tomograms.
        This is determined by the shape of the object_xs array.

        The equations and their derivation is found in:
        Turbell, H. (2001). Cone-Beam Reconstruction Using Filtered Backprojection. Science And Technology.
        Retrieved from http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.134.5224&amp;rep=rep1&amp;type=pdf

        Parameters
        ----------
        object_xs : np.ndarray
            The x-coordinate array of the object to be reconstructed
        object_ys : np.ndarray
            The x-coordinate array of the object to be reconstructed
        object_zs : np.ndarray
            The x-coordinate array of the object to be reconstructed
        settings : obj
            The settings object containing all necessary settings for the reconstruction


        Returns
        -------
        detector_cords_a
            The detector coordinates corresponding to the given points
        detector_cords_b
            The detector coordinates corresponding to the given points

        """

    detector_cords_a = (((object_ys * settings.source_to_detector_dist) / (object_xs + settings.source_to_object_dist)) -
          settings.detector_us[0]) / settings.pixel_size_u

    if object_xs.ndim == 2:
        detector_cords_b = (((object_zs[np.newaxis, np.newaxis, :] * settings.source_to_detector_dist) / (
                object_xs[:, :, np.newaxis] + settings.source_to_object_dist)) - settings.detector_vs[
                  0]) / settings.pixel_size_v

    elif object_xs.ndim == 1:
        detector_cords_b = (((object_zs[np.newaxis, :] * settings.source_to_detector_dist) / (
                object_xs[:, np.newaxis] + settings.source_to_object_dist)) - settings.detector_vs[
                  0]) / settings.pixel_size_v
    else:
        raise IOError("Invalid dimensions on the object coordinates")

    return detector_cords_a, detector_cords_b


def fdk_axisym(projection, settings):

    proj_width, proj_height = projection.shape
    proj_center = int(proj_width / 2)

    # Allocate an empty array
    recon_slice = np.zeros((proj_center, proj_height), dtype=np.float)

    for frame_nr, angle in enumerate(settings.projection_angs):
        print("Back projecting frame nr: %i" % frame_nr)

        # Calculate the coordinates of the slice for a given rotation
        x_rotated, y_rotated = rotate_coordinates(np.zeros_like(settings.object_xs)[:proj_center],
                                                  settings.object_ys[:proj_center],
                                                  np.radians(angle))

        # Correct for a center of rotation not being at y=0
        y_rotated += settings.center_of_rot_y

        detector_cords_a, detector_cords_b = map_object_to_detector_coords(x_rotated, y_rotated, settings.object_zs,
                                                                           settings)

        # a is independent of Z but has to match the shape of b
        detector_cords_a = detector_cords_a[:, np.newaxis] * np.ones_like(detector_cords_b)

        # This term is caused by the divergent cone geometry
        ratio = (settings.source_to_object_dist ** 2.) / (settings.source_to_object_dist + x_rotated) ** 2.

        recon_slice = recon_slice + ratio[:, np.newaxis] * map_coordinates(projection,
                                                                           [detector_cords_a, detector_cords_b],
                                                                           cval=0., order=1)

    return recon_slice / np.float(settings.n_projections)


def fdk(projection, param):
    """Filtered back projection algorithm as proposed by Feldkamp David Kress, adapted for axisymmetry.

        This implementation has been adapted for axis-symmetry by using a single projection only and
        by only reconstructing a single R-Z slice.


        This algorithm is based on:
        https://doi.org/10.1364/JOSAA.1.000612

        but follows the notation used by:
        Turbell, H. (2001). Cone-Beam Reconstruction Using Filtered Backprojectionn. Science And Technology.
        Retrieved from http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.134.5224&amp;rep=rep1&amp;type=pdf

        Parameters
        ----------
        projection : np.ndarray
            The projection used in the reconstruction
        settings : obj
            The settings object containing all neccessary settings for the reconstruction


        Returns
        -------
        ndarray
            The reconstructed slice

            The reconstructed slice is a R-Z plane of a axis-symmetric tomogram where Z is the symmetry axis.

        """

    if not isinstance(param, Config):
        raise IOError("Only instances of Param are valid settings")

    if type(projection) != np.ndarray:
        raise IOError("The projections have to be in a numpy ndarray")


    projection = -np.log(projection)

    if projection.ndim == 2:
        projection_filtered = ramp_filter_and_weight(projection[:, :, np.newaxis], param)
    else:
        raise IOError("The projection has to be a 2D array")


    img = fdk_axisym(projection_filtered[:, :, 0], param)
    return img



from unittest import TestCase
from axitom.config import Config
from axitom.backprojection import map_object_to_detector_coords
import numpy as np


class TestMap_object_to_detector_coords(TestCase):

    def make_default_config(self):
        config = Config()
        config.object_size_x = 200.
        config.object_size_y = 200.
        config.object_size_z = 200.
        config.detector_size_u = config.object_size_x
        config.detector_size_v = config.object_size_y

        config.n_pixels_u = 20
        config.n_pixels_v = 20
        config.n_voxels_x = 20
        config.n_voxels_y = 20
        config.n_voxels_z = 20
        config.update()
        return config

    def test_map_object_to_detector_coords_1xmag(self):
        """
        When the detector and object are located at the same distance from the source,
        the magnification should be 1x and the coordinates should be identical.
        """
        # "zero" magnification test
        tol = 1e-6
        # This should yield zero magnification and equal coordinates
        config = self.make_default_config()
        config.source_to_detector_dist = 1.e10
        config.source_to_object_dist = 1.e10
        config.update()

        xs, ys = np.meshgrid(config.object_xs, config.object_ys)
        zs = config.object_zs
        detector_a, detector_b = map_object_to_detector_coords(xs, ys, zs, config)

        corr_us = config.n_pixels_u * (config.detector_us - config.detector_us[0]) / config.detector_size_u
        corr_vs = config.n_pixels_v * (config.detector_vs - config.detector_vs[0]) / config.detector_size_v

        xs_error = np.abs(detector_a[:, :] - corr_us[:, np.newaxis])

        ys_error = np.abs(detector_b[:, :, :].transpose() - corr_vs[:, np.newaxis, np.newaxis])

        if np.max(xs_error) > tol or np.max(ys_error) > tol:
            print("Largest error: ",xs_error.max())
            print("at positions: ",np.argmax(xs_error))

            print("Largest error: ",ys_error.max())
            print("at positions: ",np.argmax(ys_error))

            self.fail()

    def test_map_object_to_detector_coords_2xmag(self):
        """
        When the source to detector distance is twice the source to object distance,
        the magnification should be 2x.
        """
        tol = 1e-6
        # This should yield zero magnification and equal coordinates
        config = self.make_default_config()
        config.object_size_x = config.detector_size_u / 2
        config.object_size_y = config.detector_size_v / 2
        config.object_size_z = config.detector_size_v / 2

        config.source_to_object_dist = 1.e10
        config.source_to_detector_dist = 2. * config.source_to_object_dist

        config.update()

        xs, ys = np.meshgrid(config.object_xs, config.object_ys)
        detector_a, detector_b = map_object_to_detector_coords(xs, ys, config.object_zs, config)

        corr_us = config.n_pixels_u * (config.detector_us - config.detector_us[0]) / config.detector_size_u
        corr_vs = config.n_pixels_v * (config.detector_vs - config.detector_vs[0]) / config.detector_size_v

        xs_error = np.abs(detector_a[:, :] - corr_us[:, np.newaxis])
        ys_error = np.abs(detector_b[:, :, :].transpose() - corr_vs[:, np.newaxis, np.newaxis])

        if np.max(xs_error) > tol or np.max(ys_error) > tol:
            # print(xs_error)
            self.fail()



from unittest import TestCase
from axitom.phantoms import barrel
from axitom.projection import axis_sym_projection
import numpy as np
import os
import axitom



class TestAxis_sym_projection(TestCase):
    """
    For a parallel beam, the radon transform is just the sum of values along an axis.
    We can mimic this by setting the source to specimen and source to detector distances equal and very large.
    """
    def test_axis_sym_projection(self):
        tol = 1e-6

        dir_path = os.path.dirname(os.path.realpath(__file__))
        param = axitom.config_from_xtekct(dir_path + "/example_data/settings.xtekct")
        param.n_voxels_x = 100
        param.n_voxels_y = 100
        param.n_voxels_z = 100
        #
        param.object_size_x = 100.
        param.object_size_y = 100.
        param.object_size_z = 100.

        param.n_pixels_u = 100
        param.n_pixels_v = 100

        param.detector_size_u = param.object_size_x
        param.detector_size_v = param.object_size_y

        param.source_to_detector_dist = 1.0e20
        param.source_to_object_dist = 1.e20

        param.update()

        volume = barrel(100)

        angles = [0.]

        proj = axis_sym_projection(volume, param, angles=angles)

        error = np.abs(np.sum(volume, axis=1) - proj)

        if np.max(error)>tol:
            self.fail()



import numpy as np
from scipy.ndimage import map_coordinates, rotate
from copy import copy


def forward_project(data, param, angle_deg):

    # Rotate the data3d around the third axis

    data3d = rotate(data, angle_deg, (1, 0), reshape=False)

    # Now integrate along the rays
    uu, vv = np.meshgrid(param.detector_us, param.detector_vs)

    Ratios = (param.object_ys + param.source_to_object_dist) / param.source_to_detector_dist

    pus = (uu[:, np.newaxis, :] * Ratios[np.newaxis, :, np.newaxis] - param.object_xs[0]) / param.voxel_size_x
    pvs = (vv[:, np.newaxis, :] * Ratios[np.newaxis, :, np.newaxis] - param.object_zs[0]) / param.voxel_size_z
    pys = np.arange(pus.shape[1])[np.newaxis, :, np.newaxis] * np.ones_like(pvs)

    proj2d = np.sum(map_coordinates(data3d[:, :, :], [pvs[:, :, :],
                                                      pys, pus[:, :, :]], cval=0.,order=3), axis=1)

    dist = np.sqrt(param.source_to_detector_dist ** 2. + uu ** 2. + vv ** 2.) / (
        param.source_to_detector_dist) * param.voxel_size_y

    return proj2d * dist


def axis_sym_projection(volume, param, angles=None):
    """ Projection of a volume onto a sensor plane using a conical beam geometry

        This implementation has been adapted for axis-symmetry and returns a single projection only

        Parameters
        ----------
        volume : np.ndarray
            The volume which will be projected onto the sensor
        param : obj
            The settings object
        angles : list
            The angles which will be used for the forward projection

        Returns
        -------
        ndarray
            The projection

        """
    modified_param = copy(param)
    if angles is not None:
        modified_param.projection_angs = angles
        modified_param.nProj = len(modified_param.projection_angs)
    else:
        modified_param.projection_angs = np.linspace(0., 45., param.n_voxels_x / 2)
        modified_param.nProj = len(modified_param.projection_angs)

    radiogram = np.zeros((param.n_pixels_u, param.n_pixels_v), dtype=np.float)
    for i , angle_deg in  enumerate(modified_param.projection_angs):
        print("Projecting fram nr: ", i)
        radiogram += forward_project(volume, modified_param, angle_deg)
    return radiogram / np.float(modified_param.nProj)



import numpy as np
import axitom
from unittest import TestCase
from scipy.ndimage.filters import median_filter
import os


def run_reconstruction():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    config = axitom.config_from_xtekct(dir_path + "/example_data/settings.xtekct")
    radiogram = axitom.read_image(dir_path + "/example_data/radiogram.tif", flat_corrected=True)

    # Remove some edges that are in field of view
    radiogram[:250, :] = 0.95
    radiogram[1800:, :] = 0.95

    radiogram = median_filter(radiogram, size=20)

    _, center_offset = axitom.object_center_of_rotation(radiogram, config, background_internsity=0.9)
    config.center_of_rot_y = center_offset

    config.update()

    reconstruction = axitom.fdk(radiogram, config)

    return reconstruction


def normalize_grey_scales(image):
    undeformed_grey_scale = np.average(image[250:500, 0:250])
    background_grey_scale = np.average(image[870:1020, 280:375])

    return (image - background_grey_scale) / (undeformed_grey_scale - background_grey_scale)


class Test_CompareToExternalSoftware(TestCase):
    def test_compare_to_external_software(self):
        """
        Compare the results obtained by using only a single radiogram from a full dataset to the results
        obtained by using the whole dataset and external reconstruction software. The reconstruction by
        the external software is done in advance and only the result is used here.
        """
        dir_path = os.path.dirname(os.path.realpath(__file__))

        self.tol = 2e-2
        recon = run_reconstruction()
        recon_crop = recon.transpose()[:, ::-1][1:, :400]
        recon_crop_norm = normalize_grey_scales(recon_crop)

        correct = axitom.read_image(dir_path + "/example_data/recon_by_external_software.tif")
        correct_norm = normalize_grey_scales(correct.transpose())

        error_field = np.abs(recon_crop_norm[::-1, :] - correct_norm)

        if np.average(error_field) > self.tol:
            self.fail(
                "The reconstruction did not match the reconstruction performed by another software and the error was: %f" % np.average(
                    error_field))



import scipy.signal as sig
import numpy as np


def ramp_kernel_real(cutoff, length):
    """Ramp filter kernel in real space defined by the cut-off frequency and the spatial dimension

        Parameters
        ----------
        cutoff : float
            The cut-off frequency
        length : int
            The kernel filter length

        Returns
        -------
        ndarray
            The filter kernel


        """
    pos = np.arange(-length, length, 1)
    return cutoff ** 2.0 * (2.0 * np.sinc(2 * pos * cutoff) - np.sinc(pos * cutoff) ** 2.0)


def add_weights(projections, settings):
    """Add weights to the projections according to the ray length traveled through a voxel

        Parameters
        ----------
        projection : np.ndarray
            The projection used in the reconstruction
        settings : obj
            The settings object containing all necessary settings for the reconstruction

        Returns
        -------
        ndarray
            The projections weighted by the ray length


        """
    uu, vv = np.meshgrid(settings.detector_us, settings.detector_vs)

    weights = settings.source_to_detector_dist / np.sqrt(
        settings.source_to_detector_dist ** 2. + uu ** 2. + vv ** 2.)

    return projections * weights[:, :, np.newaxis]


def ramp_filter_and_weight(projections, settings):
    """Add weights to the projections and apply a ramp-high-pass filter set to 0.5*Nyquist_frequency

        Parameters
        ----------
        projection : np.ndarray
            The projection used in the reconstruction
        settings : obj
            The settings object containing all necessary settings for the reconstruction

        Returns
        -------
        ndarray
            The projections weighted by the ray length and filtered by ramp filter

        """
    projections_weighted = add_weights(projections, settings)

    n_pixels_u, _, _ = np.shape(projections_weighted)
    ramp_kernel = ramp_kernel_real(0.5, n_pixels_u)

    projections_filtered = np.zeros_like(projections_weighted)

    _, n_lines, n_projections = projections_weighted.shape

    for i in range(n_projections):
        for j in range(n_lines):
            projections_filtered[:, j, i] = sig.fftconvolve(projections_weighted[:, j, i], ramp_kernel, mode='same')

    scale_factor = 1. / settings.pixel_size_u * np.pi * (
            settings.source_to_detector_dist / settings.source_to_object_dist)

    return projections_filtered * scale_factor



from unittest import TestCase
import numpy as np
from axitom.backprojection import rotate_coordinates

class Test_RotateCoordinates(TestCase):
    def test_rotate_coordinates_90deg(self):
        tol = 1e-9
        n_coordinates = 10
        rotation_angle = np.pi/2.
        xs, ys = np.meshgrid(np.arange(n_coordinates),np.arange(n_coordinates))
        xr,yr = rotate_coordinates(xs,ys,rotation_angle)

        error_xs = np.abs(xr-ys)
        error_ys = np.abs(yr+xs)

        if np.max(error_xs) >tol or np.max(error_ys) >tol:
            print("The maximum error in X after rotation was:",np.max(error_xs) )
            print("The maximum error in Y after rotation was:",np.max(error_ys) )
            self.fail()


    def test_rotate_coordinates_0deg(self):
        tol = 1e-9
        n_coordinates = 10
        rotation_angle = 0.0
        xs, ys = np.meshgrid(np.arange(n_coordinates),np.arange(n_coordinates))
        xr,yr = rotate_coordinates(xs,ys,rotation_angle)

        error_xs = np.abs(xr-xs)
        error_ys = np.abs(yr-ys)

        if np.max(error_xs) >tol or np.max(error_ys) >tol:
            print("The maximum error in X after rotation was:",np.max(error_xs) )
            print("The maximum error in Y after rotation was:",np.max(error_ys) )
            self.fail()

    def test_rotate_coordinates_forward_and_reverse(self):
        tol = 1e-9
        n_coordinates = 10
        rotation_angle = np.pi/4
        xs, ys = np.meshgrid(np.arange(n_coordinates),np.arange(n_coordinates))
        xr_forw,yr_forw = rotate_coordinates(xs,ys,rotation_angle)
        xr,yr = rotate_coordinates(xr_forw,yr_forw,-rotation_angle)


        error_xs = np.abs(xr-xs)
        error_ys = np.abs(yr-ys)

        if np.max(error_xs) >tol or np.max(error_ys) >tol:
            print("The maximum error in X after rotation was:",np.max(error_xs) )
            print("The maximum error in Y after rotation was:",np.max(error_ys) )
            self.fail()




import numpy as np
from .parse import parse_xtekct_file


class Config(object):

    def __init__(self):
        """ Configuration object which contains all settings neccessary for the forward projection
            and tomographic reconstruction using the axitom algorithm.

        """

        self.n_voxels_x = 2000
        self.n_voxels_y = 2000
        self.n_voxels_z = 2000

        self.object_size_x = 27.229675726
        self.object_size_y = 27.229675726
        self.object_size_z = 27.229675726

        self.n_pixels_u = 2000
        self.n_pixels_v = 2000

        self.detector_size_u = 400.
        self.detector_size_v = 400.
        self.source_to_detector_dist = 1127.464
        self.source_to_object_dist = 76.7511959075928
        self.angular_inc = 1.

        self.pixel_offset_u = 0
        self.pixel_offset_v = 0

        self.center_of_rot_y = 0.0

        self.projection_angs = np.arange(0., 360, self.angular_inc)
        self.n_projections = len(self.projection_angs)

        self.voxel_size_x = self.object_size_x / self.n_voxels_x
        self.voxel_size_y = self.object_size_y / self.n_voxels_y
        self.voxel_size_z = self.object_size_z / self.n_voxels_z

        self.pixel_size_u = self.detector_size_u / self.n_pixels_u
        self.pixel_size_v = self.detector_size_v / self.n_pixels_v

        self.object_xs = (np.arange(self.n_voxels_x, dtype=np.float64) - self.n_voxels_x / 2.) * self.voxel_size_x
        self.object_ys = (np.arange(self.n_voxels_y, dtype=np.float64) - self.n_voxels_y / 2.) * self.voxel_size_y
        self.object_zs = (np.arange(self.n_voxels_z, dtype=np.float64) - self.n_voxels_z / 2.) * self.voxel_size_z

        self.detector_us = (np.arange(self.n_pixels_u,
                                      dtype=np.float64) - self.n_pixels_u / 2.) * self.pixel_size_u + self.pixel_offset_u * self.pixel_size_u
        self.detector_vs = (np.arange(self.n_pixels_v,
                                      dtype=np.float64) - self.n_pixels_v / 2.) * self.pixel_size_v + self.pixel_offset_v * self.pixel_size_v

    def update(self):
        self.projection_angs = np.arange(0., 360, self.angular_inc)
        self.n_projections = len(self.projection_angs)

        self.voxel_size_x = self.object_size_x / self.n_voxels_x
        self.voxel_size_y = self.object_size_y / self.n_voxels_y
        self.voxel_size_z = self.object_size_z / self.n_voxels_z

        self.pixel_size_u = self.detector_size_u / self.n_pixels_u
        self.pixel_size_v = self.detector_size_v / self.n_pixels_v

        self.object_xs = (np.arange(self.n_voxels_x, dtype=np.float64) - self.n_voxels_x / 2.) * self.voxel_size_x
        self.object_ys = (np.arange(self.n_voxels_y, dtype=np.float64) - self.n_voxels_y / 2.) * self.voxel_size_y
        self.object_zs = (np.arange(self.n_voxels_z, dtype=np.float64) - self.n_voxels_z / 2.) * self.voxel_size_z

        self.detector_us = (np.arange(self.n_pixels_u,
                                      dtype=np.float64) - self.n_pixels_u / 2.) * self.pixel_size_u + self.pixel_offset_u * self.pixel_size_u
        self.detector_vs = (np.arange(self.n_pixels_v,
                                      dtype=np.float64) - self.n_pixels_v / 2.) * self.pixel_size_v + self.pixel_offset_v * self.pixel_size_v


def config_from_xtekct(file_path):
    """ Make config object from a Nikon X-tek CT input file

        The .xtekct file is parsed and a config file containing all relevant settings is returned.


        Parameters
        ----------
        file_path : str
            The path to the .xtekct file

        Returns
        -------
        obj
            Config object

        """

    inputfile = parse_xtekct_file(file_path)
    conf = Config()

    try:
        conf.n_voxels_x = inputfile["VoxelsX"]
        conf.n_voxels_y = inputfile["VoxelsY"]
        conf.n_voxels_z = inputfile["VoxelsZ"]

        conf.object_size_x = inputfile["VoxelSizeX"] * conf.n_voxels_x
        conf.object_size_y = inputfile["VoxelSizeY"] * conf.n_voxels_y
        conf.object_size_z = inputfile["VoxelSizeZ"] * conf.n_voxels_z

        conf.n_pixels_u = inputfile["DetectorPixelsX"]
        conf.n_pixels_v = inputfile["DetectorPixelsY"]

        conf.detector_size_u = inputfile["DetectorPixelSizeX"] * conf.n_pixels_u
        conf.detector_size_v = inputfile["DetectorPixelSizeY"] * conf.n_pixels_v

        conf.source_to_detector_dist = inputfile["SrcToDetector"]
        conf.source_to_object_dist = inputfile["SrcToObject"]
    except Exception as e:
        raise IOError("Parsing of X-tec file failed with key: ", e)

    return conf



import numpy as np


def barrel(domain_size=128, outer_rad_fraction=0.7,center_val=None):
    """ Barrel shaped phantom with a linear density gradient
    The domain size is cubic with dimension "domain_size" along each size

    Parameters
    ----------
    domain_size : int
        The length of the sides of the domain
    outer_rad_fraction : float
        The diameter of the barrel given as a the fraction of the side length
    center_val : float
        The density value in the center of the barrel

    Returns
    -------
    ndarray
        The phantom
    """

    center = domain_size / 2.
    domain = np.zeros((domain_size, domain_size, domain_size), dtype=np.float64)
    xs, ys = np.meshgrid(np.arange(domain_size), np.arange(domain_size))
    xs = xs - center
    ys = ys - center
    r = np.sqrt(xs ** 2. + ys ** 2.)
    domain[r < outer_rad_fraction * center, :] = 1.
    if center_val is not None:
        domain = domain * (center_val + (r / (outer_rad_fraction * center)) ** 2. * 0.5)[:, :, np.newaxis]
    return domain






def parse_xtekct_file(file_path):
    """Parse a X-tec-CT file into a dictionary
    Only = is considered valid separators

        Parameters
        ----------
        file_path : string
            The path to the file to be parsed

        Returns
        -------
        string
            A dictionary containing all key-value pairs found in the X-tec-CT input file
        """
    myvars = {}
    with open(file_path) as myfile:
        for line in myfile:
            name, var = line.partition("=")[::2]
            try:
                if "." in var:
                    myvars[name.strip()] = float(var)
                else:
                    myvars[name.strip()] = int(var)
            except ValueError:
                myvars[name.strip()] = var

    return myvars


import numpy as np
import natsort
import os
from imageio import imread


def find_center_of_gravity_in_radiogram(radiogram, background_internsity=0.9):
    """ Find axis of rotation in the radiogram.
        This is done by binarization of the image into object and background
        and determining the center of gravity of the object.

        Parameters
        ----------
        radiogram : ndarray
            The radiogram, normalized between 0 and 1
        background_internsity : float
            The background intensity threshold


        Returns
        -------
        float64
            The center of gravity in the u-direction
        float64
            The center of gravity in the v-direction

        """
    n, m = np.shape(radiogram)
    xs, ys = np.meshgrid(np.arange(m), np.arange(n))
    covered_pixels = np.zeros_like(radiogram,dtype=np.float)
    covered_pixels[radiogram < background_internsity] = 1.

    # Determine center of gravity
    center_of_grav_x = np.average(np.sum(xs * covered_pixels, axis=1) / np.sum(covered_pixels, axis=1)) - m / 2.
    center_of_grav_y = np.average(np.sum(ys * covered_pixels, axis=0) / np.sum(covered_pixels, axis=0)) - n / 2.
    return center_of_grav_x, center_of_grav_y


def object_center_of_rotation(radiogram, param, background_internsity=0.9, method="center_of_gravity"):
    """ Find the axis of rotation of the object pictures in the radiogram
        This is done by determining the center of rotation of the radiogram and scaling the coordinates
        to object coordinates

        Parameters
        ----------
        radiogram : ndarray
            The radiogram, normalized between 0 and 1
        param : object
            The parameters used for the tomographic reconstruction
        background_internsity : float
            The background intensity threshold
        method : string
            The background intensity threshold


        Returns
        -------
        float64
            The center of gravity in the x-direction
        float64
            The center of gravity in the y-direction

        """
    if radiogram.ndim != 2:
        raise ValueError("Invalid radiogram shape. It has to be a 2d numpy array")

    if method == "center_of_gravity":
        center_x,center_y = find_center_of_gravity_in_radiogram(radiogram,background_internsity)
    else:
        raise ValueError("Invalid method")

    scale = (param.source_to_object_dist / param.source_to_detector_dist) * param.pixel_size_u

    return scale * center_x, scale * center_y


def rotate_coordinates(xs_array, ys_array, angle_rad):
    """ Rotate coordinate arrays by a given angle

        Parameters
        ----------
        xs_array : ndarray
            Two dimensional coordinate array with x-coordinates
        ys_array : ndarray
            Two dimensional coordinate array with y-coordinates
        angle_rad : float
            Rotation angle in radians

        Returns
        -------
        ndarray
            The rotated x-coordinates
        ndarray
            The rotated x-coordinates

        """
    rx = (xs_array * np.cos(angle_rad) + ys_array * np.sin(angle_rad))
    ry = (-xs_array * np.sin(angle_rad) + ys_array * np.cos(angle_rad))
    return rx, ry


def list_files_in_folder(path, file_type=".tif"):
    """ List all files with a given extension for a given path. The output is sorted

        Parameters
        ----------
        path : str
            Path to the folder containing the files
        file_type : str
            The file extension ex. ".tif"

        Returns
        -------
        list
            A list of sorted file names

        """
    return natsort.natsorted([file for file in os.listdir(path) if file.endswith(file_type)])


def shading_correction(radiograms, flats, darks):
    """ Perform shading correction on a a list of radiograms based on a list of flat images and list of dark images.

        The correction is: corrected = (radiogram-dark)/(flat-dark)

        Parameters
        ----------
        radiograms : list
            A list of radiograms that will be corrected
        flats : list
            A list of flat images
        darks : list
            A list of dark images


        Returns
        -------
        list
            A list of corrected radiograms

        """
    flat_avg = np.array(flats)
    flat_avg = np.average(flat_avg, axis=0)

    dark_avg = np.array(darks)
    dark_avg = np.average(dark_avg, axis=0)

    corrected_radiograms = []
    for radiogram in radiograms:
        corrected_radiograms.append((radiogram - dark_avg) / (flat_avg - dark_avg))
    return corrected_radiograms


def read_image(file_path, flat_corrected=False):
    """ Read an image specified by the file_path
        This function allways returns a float64 single channel image

        Parameters
        ----------
        file_path : str
            Path to the file

        Returns
        -------
        ndarray
            The single channel image with float64 values

        """
    image = imread(file_path).astype(np.float64)
    if image.ndim == 3:
        image =  np.average(image, axis=2)
    if flat_corrected:
        image = image/image.max()
    return image.transpose()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="axitom",
    version="0.1.1",
    author="PolymerGuy",
    author_email="sindre.n.olufsen@ntnu.no",
    description="This python package provides tools for axis-symmetric cone-beam computed tomography",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PolymerGuy/AXITOM",
    include_package_data=True,
    packages=setuptools.find_packages(),
    install_requires=[
            'numpy',
            'scipy',
            'pytest',
            'coverage',
            'pytest-cov',
            'codecov',
            'matplotlib',
            'scikit-image',
            'natsort',
            'imageio'
      ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import numpy as np
import axitom
from axitom.phantoms import barrel
from unittest import TestCase
import os


class Test_SimulatedProjections(TestCase):

    def run_reconstruction_and_compare(self, path_to_projections, correct_body):
        # Set the work path as the filepath to this file
        dir_path = os.path.dirname(os.path.realpath(__file__))

        # Set the same settings as used in the forward projection
        param = axitom.config_from_xtekct(dir_path + "/example_data/settings.xtekct")
        param.n_voxels_x = 500
        param.n_voxels_y = 500
        param.n_voxels_z = 500
        param.n_pixels_u = 500
        param.n_pixels_v = 500
        param.update()

        proj = np.load(dir_path + path_to_projections)
        proj = np.exp(-proj)

        reconstruction = axitom.fdk(proj, param)

        # Note that the reconstruction give the radial slice only
        deviation_field = np.abs(correct_body[:250, 250, :] - reconstruction)

        # Sample some deviation values from the background and the interior
        background_deviation = deviation_field[5:60, 75:-75]
        interior_deviation = deviation_field[90:, 75:-75]

        if np.max(background_deviation) > self.tol:
            self.fail("The background was not reconstructed properly with largest deviation of %f" % np.max(
                background_deviation))

        if np.max(interior_deviation) > self.tol:
            self.fail(
                "The interior was not reconstructed properly with largest deviation of %f" % np.max(interior_deviation))

    def test_reconstruction_of_massive_barrel(self):
        """
        Check if the reconstruction of a massive barrel matches the volume model from which
        the radiograms were produced.
        """

        self.tol = 2e-2
        path_to_projections = r"/example_data/barrel_projections_axisym.npy"
        correct = barrel(500)
        self.run_reconstruction_and_compare(path_to_projections, correct)

    def test_reconstruction_of_gradient_barrel(self):
        """
        Check if the reconstruction of a barrel with a radial density gradient matches the volume model from which
        the radiograms were produced.
        """
        self.tol = 2e-2
        path_to_projections = r"/example_data/barrel_grad_projections_axisym.npy"
        correct = barrel(500, center_val=0.5)
        self.run_reconstruction_and_compare(path_to_projections, correct)
